#ifndef CMD_H
#define CMD_H
/** \file
   Definitions for various user interface (command) stuff.
*/
/*   

   Copyright  1992-2009 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16, all ri

   $Log: cmd.h,v $
   Revision 1.2.2.1  2005/03/15 14:57:09  kraemer
   *** empty log message ***

   Revision 1.2  2000/04/03 12:35:01  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1999/02/02 20:13:13  kraemer


*/

#include <limits.h>

#include "u.h"
#include "cmdsys.h"
#include "cmdintuition.h"
#include "cmdprtcl.h"
#include "cmditem.h"
#include "cmdalias.h"
#include "ipar.h"

#if defined(CMDUNIX) || defined(__EMX__)
char *readline( char *);    /* readline from GNU */
#endif

                                      /** \struct SCMDDEF
                                          \brief for command definition.
                                      */ 
struct SCMDDEF { 
   const char *pcCmd;                 /**< command string */   
   int (*eCmd)();                     /**< entry for command processor */  
   unsigned long ulFlag;              /**< command flags */ 
};
                                       
#define CMDDEF_HIDE        0x00010000 /* mark a "hidden" command */
#define CMDDEF_WARNUNKNOWN 0x00020000 /* unknown parameter warn only */
#define CMDDEF_INTERNAL    0x00040000 /* an internal command */

/*
   some platform specific definitions ...
*/
                                      /* special for X ... */
#if defined(CMDUNIX)

#include <Xm/Text.h>
#include <Xm/MainW.h>
#include <Xm/Command.h>

                                      /** \struct SCMDX
                                          \brief interfacing w/ X.
                                      */ 
struct SCMDX {
   Widget wdgTop;                     /**< top widget */
   Widget wdgMain;                    /**< main widget */
   XtAppContext xtApp;                /**< app context */
};
#endif
                                      /* special for PM ... */
#if defined(__EMX__)

#define CMDPMSCROLLCLASS "cmdscroll"
                                      /** \struct SCMDPM 
                                          \brief interfacing w/ PM .
                                      */ 
struct SCMDPM {
   HAB hab;                           /**< PM anchor block */
   HMQ hmq;                           /**< PM message queue */
   HWND hWndClient, hWndFrame;        /**< client window */
   PFNWP pfnwpFrameOld, pfnwpClientOld; /**< */   
   FONTMETRICS     fm;                /**< font metrics */
   FRAMECDATA fcdata;                 /**< frame window stuff */ 
   POINTL sPPos;                      /**< "cursor" position */   
};
#endif
                                      /* special for Amiga Intuition */
#if defined(AMIGA)

#define ANCHOR

                                      /** \struct SCMDINTUITION
                                          \brief interfacing w/ Intuition.
                                      */ 
struct SCMDINTUITION {
   struct Window *psWindow;           /**< current window */
   struct Task *psTask;               /**< current task */
   struct Process *psProcess;         /**< current process */
   struct MsgPort *psMP;              /**< for console I/O */
   struct FileHandle *psFHIn, *psFHOut;/**< file handles */
   struct DosPacket *psDPRead, *psDPWrite;/**< DOS packets */
   void   *pvVI;                      /**< visual info */
   struct Menu *psMenu;               /**< save menu strip here */   
#if defined(ANCHOR)
#else
   unsigned long ulSignalMask;        /**< logical OR of all sigbits */
   int (*eWindow)( struct IntuiMessage *);/**<  */
#endif
};    
#endif

                                      /** \struct SCMD
                                          \brief base structure for all command activity.
                                      */ 
struct SCMD {
   unsigned long ulFlag;              /**< global system flags */
   unsigned long ulExit;              /**< exit code */
   int iRCCmd, iPad;                  /**< RC from last command */
   long mLevel, lLevel;               /**< recursion level for EXEC cmd */
   long mCmds,  lCmds;                /**< max # of commands for EXEC cmd */
   unsigned long ulElapsedTime;       /**< elapsed time (sec.s) */
   unsigned long ulElapsedTimeAccu;   /**<  */
   unsigned long ulElapsedBuf[2];     /**< buffer for elapsed time query */
   unsigned long ulCPUTime;           /**< CPU time (sec.s) */
   unsigned long ulCPUTimeAccu;       /**<  */
   unsigned long ulCPUBuf[2];         /**< buffer for CPU time query */
   char *pcPrompt;                    /**< command prompt */
#if defined(AMIGA)
   struct SCMDINTUITION sI;           /**< Amiga Intuition interface */
#endif
#if defined(__EMX__)
   struct SCMDPM sPM;                 /**< OS/2 PM interface */
#endif
#if defined(CMDUNIX)
   struct SCMDX sX;                   /**< X interface */
#endif
   struct SCMDITEMLIST sCIL;          /**< command list head */
   struct SIPARLIST sIPL;             /**< IPAR list head */
   struct SCMDALIAS sCA;              /**< alias list */
   struct SCMDPRTCL sCP;              /**< protocol */
   struct SCMDERR sCE;                /**< holds error info */
};
                                      /* */
#define CMDEXEC_MAXLEVEL 10
#define CMDEXEC_MAXCMDS  LONG_MAX
                                      /* special command codes */ 
enum { CMD_EXIT=1, CMD_QUIT=2, CMD_CLOSE=3, };
                                      /* cmd status flags */
#define CMDFLAG_GUI       0x00000001  /* we run w/ GUI */ 
#define CMDFLAG_TIMEON    0x00000002  /* report time consumed */
#define CMDFLAG_KEEPPRTCL 0x00000004  /* flag to keep protocol */
#define CMDFLAG_PORT      0x00000008  /* we have installed own GUI ports */ 
                                     
                                      /* function specific action flags */
#define CMDSETUP_GUI      0x00010000  /* initialize GUI too */

#define CMDTERM_KEEPPRTCL 0x00020000  /* on terminate: keep protocol */

#define CMDDEC_DONTPRTCL  0x00040000  /* suppress cmd protocol */


#define CMD_MAXPAR 192                /* max # of parameters per command */

/*


  _cmdCmd     ( cmd, flag )
              o handle command prompt/reception
            9 x to test
              o to complete
                o A:
                  o checkout OS2.0 functions
                  o what about redirected I/O ?
                    o GUI-mode CLI freezes system on redirected input
                  o CTRL-C handling
                  o command echoing to protocol ?
                  o support for grafix windows
                o O:
                  o CTRL-C handling
                o X:
                  o todo
              o calls
                o _cmdalias*()
                o _cmdDecode()

  _cmdCommands( cmd, flag  )
              o define basic commands 
            7 x to test
              o to complete
                ? for GUI: define basic menu items (exit, quit, help, etc.)
              o calls
                o _iparlistConst()
                o _cmdDefine()

  _cmdFree    ( cmd, flag )
              o release minor command resources (not the whole system)
           3a o to test 
              o calls
                o _cmdaliasFree();                                
                o _cmditemlistFree();
                o _iparlistFree();

  _cmdInit    ( cmd, flag )
              o initialize
            2 x to test
              o calls
                o _cmdaliasInit()
                o _iparlistInit()
                o _cmditemlistInit()
                o _cmdprtclInit()

  _cmdInspect ( cmd, flag )
              o inspect command base
            1 x to test
              o calls
                o _cmdaliasInspect()
                ? _iparlistInspect()
                ? _cmditemlistInspect()
                o _cmdprtclInspect()

  _cmdSetup   ( cmd, flag [=GUI] )
              o setup command list 
            4 x to test
              o to complete
                o A:
                  o replace OS2.0  functions
                  o check/remember if user port already existing
                  o support for extra grafix windows
                ? O: tmpfile() instead tmpnam()
                  o X:
                ? define basic menu items (exit, quit, help, etc.)
              o calls
                o _cmdprtclSetup()
                o _iparlistSetup()
                o _iparlistConst()
                o _cmditemlistSetup()
                o _cmdaliasSetup()
                o _cmdFree()

  _cmdShell   ( cmd, str, flag )
              o Interface to allow host operating system commands
            5 x to test
              o calls
                o _cmdError()

  _cmdTerm    ( cmd, flag )
              o shutdown/release command base
            3 x to test 
              o A:
                o cleanup outstanding messages
                o support for extra grafix windows
              o calls
                o _cmdaliasFree();                                
                o _cmditemlistFree();
                o _iparlistFree();
                o _cmdprtclCopy2File();
                o _cmdprtclFree();



    
  _cmdDecode  ( cmd, str, flag )
              o decode & process line command
            8 x to test
              o to complete 
                ? don't hold lock on cmditemlist so long
                  (duplicate cmditem once found)
                x use ipar's/getenv() for conversion
                ? include quoting for char var's
                ? quotes within quotes 
                o doesn't recognize "EXIT" in exec's
                  => cmdcmdexec problem ?
                x return code from commands, where to store ?
                  (possibly usable for some $RC in EXEC)
                x should return > 0 for any kind of EXIT command
                o should reject values for TYPE(0) parameters
                o syntax:
        
                  <cmd> {
                          <reqpar> { <reqpar> | / null }
                          |        { <pospar> | / null } 
                          /        { <optpar>   / null }  
                          null
                        } 
                        
              o calls
                o _cmditem*()
                o _cmdprtcl()
                o _ipar*()  

  _cmdDefine  ( cmd, cmddef, flag )
              o define line command
            6 x to test
              o to complete
                x threadsafe lock cmditem's
                o nested aliases
                ? fully satan-compatible ( a() -> abbr() )
                o syntax:

      <cmd>[( A(<num>) C(comment) L(alias) T(<type>) )] { 
                                       <reqpar> { <reqpar> | / null }
                                     |        { <pospar> | / null } 
                                     /        { <optpar>   / null }  
                                     null
                                   } 
      <*par> = <parname>[( A(<num>) C(comment) T(<type>) L(alias) [ { R() D() } ] )] 
      <type> = 0 | b | boolean
               1 | i | short
               2 | l | long
               3 | r | float
               4 | d | double
             >=5                 char[>=5]  
                   c             char[*] 
                   f | filename  

              o calls
                o _cmditem*() 
               
*/

                                        /* function prototypes */
int _cmdCommands( struct SCMD *, unsigned long );
int _cmdFree    ( struct SCMD *, unsigned long );
int _cmdInit    ( struct SCMD *, unsigned long );
int _cmdInspect ( struct SCMD *, unsigned long );
int _cmdSetup   ( struct SCMD *, unsigned long );
int _cmdShell   ( struct SCMD *, const char *, unsigned long );
int _cmdTerm    ( struct SCMD *, unsigned long );
struct SCMD *_cmdTest( unsigned long );

int _cmdDecode( struct SCMD *, const char *, unsigned long );
int _cmddec   (                const char *, unsigned long );

int _cmdDefine( struct SCMD *, struct SCMDDEF *,        unsigned long );
int _cmddef   (                const char *, int (*)(), unsigned long );

int _cmdCmd  ( struct SCMD *, unsigned long );
char *_cmdget( const char *, unsigned long );

int _pyes( const char *);
int _pcfo( const char *, char *, int );
int _pcf ( const char *, char *, int );
int _pcv ( const char *, char *, int );
int _pcvo( const char *, char *, int );


int _pilo( const char *, long * );
int _pvilo( const char *, long *, int );
int _pvrl ( const char *, double *, int );


                                      /* command processors */
int __cmdcmdalias( const char *, const char *, const char *,
                   long *, long *,
                   struct SCMD *
                 );
int  _cmdcmdalias( const char *, const char *, const char *,
                   long *, long *, long *,
                   long *,
                   struct SCMD *
                 );
int _cmdcmdexec( const char *,
                 const char *, const char *, const char *, const char *,
                 const char *, const char *, const char *, const char *,
                 const char *, const char *, const char *,
                 unsigned long *, unsigned long *,
                 unsigned long *, unsigned long *,
                 struct SCMD *
               );
int _cmdcmdHelp( const char *,
                 const char *, const char *,
                 long *, long *, long *, long *,
                 long *, long *,
                 long *, long *,
                 long *, long *, const char *,
                 long *, const char *, long *,
                 long *, long *, long *,
                 long *, long *,
                 struct SCMD *
               );

int _cmdcmdexit( unsigned long *, struct SCMD * );
int _cmdcmdquit( unsigned long *, struct SCMD * );

int _cmdcmdprtcl( const char *, const char *, const char *,
                  long *, long *, long *,   
                  const char *, long *, struct SCMD * );
int _cmdcmdshell( const char *, struct SCMD * );
int _cmdcmdtime( unsigned long *, unsigned long *, unsigned long *, struct SCMD * );
int _cmdcmdtutorial( const char *, long *, long *, struct SCMD * );
int _cmdcmdwait ( long *, long *, struct SCMD * );

int _iparcmd( const char *, const char *,
              long *, long *, long *, long *,
              const char *, const char *, const char *, 
              const char *, const char *,
              long *, long *, long *, long *,  
              long *, long *,  
              long *, struct SCMD * );
int _iparcmdconst( long *, long *, long *, struct SCMD *);
int _iparcmddcl  ( const char *, const char *, const char *, const char *,
                   long *,
                   long *, struct SCMD * );
int _iparcmdgd   ( const char *, const char *, const char *,
                   long *,
                   const char *, const char *, const char *,
                   const char *, const char *, const char *,
                   const char *, const char *, const char *, 
                   long *, struct SCMD * );
int _iparcmdop   ( const char *, long *,
                   long *, struct SCMD * );
#endif
