#ifndef CMDITEM_H
#define CMDITEM_H
/** \file
   Definitions for command items (commands and command parameters).
*/
/*    

   Copyright  1992-2009 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16, 
                         all rights reserved

   $Log: cmditem.h,v $
   Revision 1.1.1.1.2.1  2005/03/15 14:57:21  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1999/02/02 20:13:14  kraemer


*/

#include "ulists.h"
#include "cmdsys.h"
#include "cmderr.h"
                                      /* global flags for commands = parameters */
#define CMDITEM_DEBUG  0x80000000     /* general check flag */
#define CMDITEM_QUIET  0x10000000     /* suppress error messages */
#define CMDITEM_NOMSG  CMDITEM_QUIET
                                      /* common item for commands & parameters */
struct SCMDITEM {
   struct Node ln;
   int nAbbr;                         /* abbreviation length */
   unsigned long ulFlag;              /* flags */
   char *pcAlias;                     /* name of alias */
   struct SCMDITEM *psCIAlias;        /* ptr. to alias */
};
                                      /* item node types */                                      
enum { CMDITEM_TYPEPAR=254, CMDITEM_TYPECMD=253, };  
                                      /*  flags */
#define CMDITEM_HIDE  0x00000002      /* a "hidden" item */
                                      /* */
/*
   Common functions
   ----------------

   _cmditemDup             : ( cito, cifr, flag )
                             o duplicate
                           4 x to test
                             o calls
                               o _cmditemInit()
                               o _cmditemFree()
   
   _cmditemFree            : ( ci, flag )
                             o free
                           3 x to test
                             o calls
                               o _cmditemInit()
   
   _cmditemInit            : ( ci, flag )
                             o init
                           2 x to test
   
   _cmditemInspect         : ( ci, flag )
                             o inspect
                           1 x to test
                             o calls
                               o _cmditemInspect()
   


   _cmditemListAbbrev      : ( lh, ci, flag )
                             o resolve alias definitions
                           5 x to test   

   _cmditemListAdd         : ( lh, ci, flag )
                             o add an item to the list
                           7 x to test
                             x inspect flag to decide cmd/par for duplicate
                             o calls
                               o _cmditemListAbbrev()
                               o _cmditem*Dup()
                               o _cmditem*Free()
   
   _cmditemListFindName    : ( lh, str, nstr, flag )
                             o locate by name, return ci *
                           6 x to test
                             x extra returns for not found/ambiguous
   
   _cmditemListResolveAlias: ( lh, flag )
                             o resolve alias definitions
                           8 x to test   
                             o calls
                               o _cmditemListFindName()


   _cmditemError           : ( ce, flag ) 
                             o error handling
                             x to test
                             
*/
                                      /* function prototypes */
int _cmditemDup    ( struct SCMDITEM *, struct SCMDITEM *, unsigned long );
int _cmditemFree   ( struct SCMDITEM *, unsigned long );
int _cmditemInit   ( struct SCMDITEM *, unsigned long );
int _cmditemInspect( struct SCMDITEM *, unsigned long );
                                       
struct SCMDITEM *_cmditemTest( unsigned long );

int _cmditemListAbbrev      ( struct List *, struct SCMDITEM *, unsigned long );
int _cmditemListAdd         ( struct List *, struct SCMDITEM *, unsigned long );
struct SCMDITEM *
    _cmditemListFindName    ( struct List *, const char *, int, unsigned long );
int _cmditemListResolveAlias( struct List *, unsigned long );

int _cmditemError  ( struct SCMDERR  *, unsigned long );


                                      /* common storage for numerical parameter values */
union UCMDPARVAL { double dd; float rr; long ll; short ii; };
                                      /* a command parameter */
struct SCMDITEMPAR {
   struct SCMDITEM sCI; 
   unsigned long ulFlag;              /* parameter flags */
   long lType;                        /* data type */
   union UCMDPARVAL uuDef;            /* default values (non-char) */
   union UCMDPARVAL uuVal;            /* actual  values (non-char) */
   char *pcDef;                       /* default values (char) */
   char *pcVal;                       /* actual  values (char) */
   void *pvVal;                       /* ptr. to actual value field */
   char *pcCom;                       /* comment text */
};
                                      /* item types */
enum {
   CMDPAR_BOOL=0, CMDPAR_SHORT, CMDPAR_LONG,
   CMDPAR_FLOAT, CMDPAR_DOUBLE,
   CMDPAR_CHARSTR, CMDPAR_FILENAME,
};
                                      /* command parameter flags */
#define CMDPAR_REPLACEDEFAULT 0x00000001  /* replaceable default */
#define CMDPAR_MASKDEF (CMDPAR_REPLACEDEFAULT)
#define CMDPAR_REQPAR         0x00000010
#define CMDPAR_POSPAR         0x00000020
#define CMDPAR_OPTPAR         0x00000040
#define CMDPAR_MASKPOS (CMDPAR_REQPAR|CMDPAR_POSPAR|CMDPAR_OPTPAR)
                                      /* */
/*

   _cmditemparDefault2Value: ( cip, flag )
                             o default => actual parameter values
                           6 x to test
                             o calls
   
   _cmditemparDup          : ( cipto, cipfr, flag )
                             o duplicate command parameter
                           4 x to test
                             o calls
                               o _cmditemparInit()
                               o _cmditemparFree()
                               o _cmditemDup()
                               
   _cmditemparFree         : ( cip, flag )
                             o release command parameter resources
                           3 x to test
                             o calls
                               o _cmditemparInit()
                               o _cmditemFree()
   
   _cmditemparFromStr      : ( cip, str, nstr, flag )
                             o command parameter definition from string 
                           5 x to test
                             o calls
                               o _cmditemparInit()
                               o _cmditemparFree()
   
   _cmditemparInit         : ( cip, flag )
                             o initialize a command parameter
                           2 x to test
                             o calls
                               o _cmditemInspect()

   _cmditemparInspect      : ( cip, flag )
                             o inspect a command parameter
                           1 x to test
                             o calls

   _cmditemparValueFromStr : ( cip, str, nstr, flag )
                             o command parameter value from string 
                           7 x to test
                             o calls

   _cmditemparValue2Default: ( cip, flag )
                             o remember actual parameter value as default 
                           8 x to test
                             o calls
   
   
*/
                                      /* function prototypes */
int _cmditemparDefault2Value( struct SCMDITEMPAR *, unsigned long );
int _cmditemparDup          ( struct SCMDITEMPAR *, struct SCMDITEMPAR *, unsigned long );
int _cmditemparFree         ( struct SCMDITEMPAR *, unsigned long ); 
int _cmditemparFromStr      ( struct SCMDITEMPAR *, const char *, int, unsigned long );
int _cmditemparInit         ( struct SCMDITEMPAR *, unsigned long ); 
int _cmditemparInspect      ( struct SCMDITEMPAR *, unsigned long ); 
int _cmditemparValue2Default( struct SCMDITEMPAR *, unsigned long );
int _cmditemparValueFromStr ( struct SCMDITEMPAR *, const char *, int, unsigned long );
                                       
struct SCMDITEMPAR *_cmditemparTest( unsigned long );

                                      /* structure for a command */
struct SCMDITEMCMD {
   struct SCMDITEM sCI; 
   unsigned long ulFlag;              /* command flags */  
   long lType;                        /* command type */
   char *pcCom;                       /* comment text */
   struct List lhPar;                 /* list header for command parameters */
   int   (*eFct)();                   /* function entry point */
   int  nParReq;                      /* total # of required parameters */
   int  nParPos;                      /* total # of positional parameters */
   int  nParOpt;                      /* total # of optional parameters */
   int  nPar;                         /* total # of parameters */
};
                                      /* command  types */
enum {
   CMDITEMCMD_ANY=0, 
   CMDITEMCMD_QUIT, CMDITEMCMD_EXIT, CMDITEMCMD_EXEC,
};

/*

   _cmditemcmdDefault2Value: ( cic, flag )
                       o   
                       o to test
                       o calls:
                         o _cmditemparDefault2Value()
   

   _cmditemcmdDup    : ( cicto, cicfr, flag )
                       o duplicate a command 
                     4 x to test
                       o calls:
                         o _cmditemcmdInit()
                         o _cmditemcmdFree()
                         o _cmditemListAdd()
                         o _cmditemListResolveAlias()
   
   _cmditemcmdFree   : ( cic, flag )
                       o release a command's resources
                     3 x to test
                       o calls:
                         o _cmditemFree()
                         o _cmditemparFree()
                         o _cmditemcmdInit()
   
   _cmditemcmdFromStr: ( cic, str, nstr, flag )
                       o command spec from string 
                     5 x to test
                       o calls:
                         o _cmditemcmdInit()
                         o _cmditemcmdFree()
   
   _cmditemcmdInit   : ( cic, flag )
                       o initialize a command 
                     2 x to test
                       o calls:
                         o _cmditemInit()
   
   _cmditemcmdInspect: ( cic, flag )
                       o inspect a command 
                     1 x to test
                       o calls:
                         o _cmditemInspect()
                         o _cmditemparInspect()
   
   _cmditemcmdValue2Default: ( cic, flag )
                       o remember actual parameter values as defaults  
                     6 x to test
                       o calls:
                         o _cmditemparValue2Default()
   

*/
                                      /* function prototypes */
int _cmditemcmdDefault2Value( struct SCMDITEMCMD *, unsigned long );
int _cmditemcmdDup          ( struct SCMDITEMCMD *, struct SCMDITEMCMD *, unsigned long );
int _cmditemcmdFree         ( struct SCMDITEMCMD *, unsigned long ); 
int _cmditemcmdFromStr      ( struct SCMDITEMCMD *, const char *, int, unsigned long );
int _cmditemcmdInit         ( struct SCMDITEMCMD *, unsigned long ); 
int _cmditemcmdInspect      ( struct SCMDITEMCMD *, unsigned long ); 
int _cmditemcmdValue2Default( struct SCMDITEMCMD *, unsigned long );

struct SCMDITEMCMD *_cmditemcmdTest( unsigned long );


                                      /* command list header */
struct SCMDITEMLIST {
   unsigned long ulFlag;              /* flag bits */
   struct List lh;
   struct SCMDLOCK sCL; 
};                

#define CMDITEMLIST_ACTIVE 0x00000001

/*

   _cmditemlistFree        : ( cilist, flag )
                             o release an item list's resources
                           3 x to test
                             o calls 
                               o _cmditemlistInit()
                               o _cmditemcmdFree()
                                  
   _cmditemlistInit        : ( cilist, flag )
                             o initialize item list
                           2 x to test
                             o calls 
   
   _cmditemlistInspect     : ( cilist, flag )
                             o inspect item list
                           1 x to test
                             o calls 
                               o _cmditemcmdInspect()
   
   _cmditemlistList        : ( cilist, ..., flag )
                             o list (for help)
                           5 x to test
                             ? alias resolution
                             o calls 
   
   _cmditemlistSetup       : ( cilist, flag )
                             o setup item list
                           4 x to test
                             o calls 
                               o _cmditemlistInit()
   
*/                          
                                      /* function prototypes */

int _cmditemlistFree   ( struct SCMDITEMLIST *, unsigned long ); 
int _cmditemlistInit   ( struct SCMDITEMLIST *, unsigned long ); 
int _cmditemlistInspect( struct SCMDITEMLIST *, unsigned long ); 
int _cmditemlistList   ( struct SCMDITEMLIST *, const char *, unsigned long ); 
int _cmditemlistSetup  ( struct SCMDITEMLIST *, unsigned long ); 

struct SCMDITEMLIST *_cmditemlistTest( unsigned long ); 

/*
   --- error codes
*/
enum { ERRCMDITEM_SYS=1,
       ERRCMDITEM_BADADDR, 
       ERRCMDITEM_SYNTAX, 
       ERRCMDITEM_CONV,
       ERRCMDITEM_BADTYPE, 
       ERRCMDITEM_BALANCE,
       ERRCMDITEM_AMBIGUOUS, 
       ERRCMDITEM_NOTFOUND,
       ERRCMDITEM_SELFALIAS, 
       ERRCMDITEM_NESTALIAS,
       ERRCMDITEM_BADNODE,
       ERRCMDITEM_BADLIST,
       ERRCMDITEM_NOTPAR,
};

#endif
