#ifndef MATHWDW_H
#define MATHWDW_H
/*
   
   Definitions for coordinate windows

   Copyright  (C) 1992-2006 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved

   $Log: mathwdw.h,v $
   Revision 1.1.2.2  2005/04/04 16:56:18  kraemer
   shift added

   Revision 1.1.2.1  2005/03/15 14:26:03  kraemer
   *** empty log message ***


*/

/*

  General
  =======

  o organization of multi-dim polygons 
      
*/

#include "u.h"
#include "m.h"
                                      /* masks */
#define MATHWDW_MASKGLOBAL 0xFF000000 /* global (e.g. debug) */
#define MATHWDW_MASKFUNC   0x00FF0000 /* function specific flags */
#define MATHWDW_MASKQUAL   0x0000FFFF /* for additional qualifiers */
                                      /* flags */
#define MATHWDW_DEBUG      0x80000000 /* general check flag */
#define MATHWDW_CHECK      MATHWDW_DEBUG  /* general check flag */
#define MATHWDW_QUIET      0x10000000 /* suppress error messages */
#define MATHWDW_NOMSG      MATHWDW_QUIET
                                      /* */
#define MATHWDWDIM 4
#define MATHWDWMOM 4
#define MATHWDWINC 16

#define MATHWDW_MAXDIM MATHWDWDIM

struct SMATHWDW {
   double dvLim[MATHWDWDIM][2];       /* rectangular limits */
                                      /* sum, mean, variance, skewness */
   double dvMom[MATHWDWDIM][MATHWDWMOM];
   unsigned long ulFlag, ulFlagInternal; /* holds flag bits */ 
   unsigned long ulType; 
   int nDim;                          /* actual dimensionality */
   int ivPntDim[2];                   /* dimensions to which pnts belong (0..ndim-1) */
   int mPnt;                          /* max # of 2D polygon points */ 
   int nPnt;                          /* current # of 2D polygon points */ 
   double **pdPnt;                    /* [nPnt][2] 2D polygon points */
};
                                      /* */
enum {
   MATHWDW_DIMX=0, MATHWDW_DIMY, MATHWDW_DIMZ, 
};
enum {
   MATHWDW_BELOW=1, MATHWDW_ABOVE=2,  
};
enum {
   MATHWDW_OUTSIDE=0, MATHWDW_INSIDE=1,  
};
                                      /* flag bits ... */
#define MATHWDW_REFCOPY 0x000000001   /* do not free() */

                                      /* function action bits ... */
#define MATHWDWINSIDE_EXCL 0x00010000
#define MATHWDWAND_EXCL    0x00010000
#define MATHWDWINIT_CLOSED 0x00010000

                                      /* macro for rectangular wdw check */
#define MATHWDWRECTINSIDEEXCL(iRC,psW,dv,mDim) \
{ int iMATHWDWDim, nMATHWDWDim;\
nMATHWDWDim = min( mDim, MATHWDWDIM );\
iRC = MATHWDW_INSIDE;\
for ( iMATHWDWDim=0; iMATHWDWDim<nMATHWDWDim; iMATHWDWDim++ ) {\
  if ( dv[iMATHWDWDim] <= (psW)->dvLim[iMATHWDWDim][0] ) { iRC = MATHWDW_OUTSIDE; break; }\
  if ( dv[iMATHWDWDim] >= (psW)->dvLim[iMATHWDWDim][1] ) { iRC = MATHWDW_OUTSIDE; break; }\
}\
}

#define MATHWDWRECTINSIDEINCL(iRC,psW,dv,mDim) \
{ int iMATHWDWDim, nMATHWDWDim;\
nMATHWDWDim = min( mDim, MATHWDWDIM );\
iRC = MATHWDW_INSIDE;\
for ( iMATHWDWDim=0; iMATHWDWDim<nMATHWDWDim; iMATHWDWDim++ ) {\
  if ( dv[iMATHWDWDim] <  (psW)->dvLim[iMATHWDWDim][0] ) { iRC = MATHWDW_OUTSIDE; break; }\
  if ( dv[iMATHWDWDim] >  (psW)->dvLim[iMATHWDWDim][1] ) { iRC = MATHWDW_OUTSIDE; break; }\
}\
}

#define MATHWDWRECTWHEREINCL(iRC,ulvWhere,psW,dv,mDim) \
{ int iMATHWDWDim, nMATHWDWDim;\
nMATHWDWDim = min( mDim, MATHWDWDIM );\
iRC = 1;\
for ( iMATHWDWDim=0; iMATHWDWDim<nMATHWDWDim; iMATHWDWDim++ ) {\
  ulvWhere[iMATHWDWDim] = 0;\
  if ( dv[iMATHWDWDim] <  (psW) -> dvLim[iMATHWDWDim][0] ) { iRC = 0; ulvWhere[iMATHWDWDim]|=MATHWDW_BELOW; }\
  if ( dv[iMATHWDWDim] >  (psW) -> dvLim[iMATHWDWDim][1] ) { iRC = 0; ulvWhere[iMATHWDWDim]|=MATHWDW_ABOVE; }\
}\
}

/*

   Functions for window handling
   =============================

   mathWdwAnd           ( psWdwTo, psWdw1, psWdw2, flag )
                        o logical and of windows, 
                          incl/exclude boundaries
                        o returns:
                          <  0: error
                          == 0: closed (no overlap)
                          >  0: opened (overlap)
                     10 x to test
                        o polygons to do 
                        o calls

   mathWdwCopy          ( psWTo, psWFr, flag )
                        o reference copy window
                     4b o to test
                        o calls
                          o _mathWdwInit() 
   
   mathWdwDup           ( psWTo, psWFr, flag )
                        o duplicate window
                      4 o OK
                        o calls
                          o _mathWdwInit() 
                          o _mathWdwFree() 
   
   mathWdwFree          ( psWdw, flag )
                      3 o OK
                        o calls
                          o _mathWdwInit() 
   
   mathWdwInit          ( psWdw, flag )
                        o initialize, "open" by default
                      2 o OK
                        o calls
   
   mathWdwInside        ( psWdw, dv, dim, flag )
                        o check inside,  including or excluding boundaries
                          returns
                          <0  for error, NaN
                          ==0 for outside
                          >0  for inside
                      8 o to test
                        x polygon check
                        x incl/excl edge
                        o calls
   
   mathWdwInspect       ( psWdw, flag )
                      1 o OK
                        o calls
   
   mathWdwOr            ( psWdwTo, psWdw1, psWdw2, flag )
                        o logical or of windows
                      6 o OK
                        o polygons to do 
                        o calls

   mathWdwPntAdd        ( mathwdw, dv[], flag )
                        o add a point to a polygon wdw
                      5 o OK
                        ? recalculate rectangular limits
                          => better after a bunch of points has been added 
                        o calls
   
   mathWdwRectFromPoly  ( mathwdw, flag )
                        o rectangular limits from polygon
                      7 x to test  
                        o calls
   
   
   mathWdwRectLineSect  ( mathwdw, pnto, dir, pnti, dim, flag )
                        o intersection w/ rectangular limits  
                      9 x to test (more tests to do ?)
                        o calls
   
   mathWdwRot           ( mwout, matrix[][], mwint, flag )
                        o rotate mathwdw 
                     11 o to test
                        o polygons todo
                        o calls
   
   mathWdwShift         ( mwout, shift[], mwint, flag )
                        o shift mathwdw, allow out==int 
                     12 x to test
                        o polygons todo
                        o calls
   

*/
                                      /* function prototypes */
int mathWdwAnd       ( struct SMATHWDW *, struct SMATHWDW *, struct SMATHWDW *, unsigned long );
int mathWdwCopy      ( struct SMATHWDW *, struct SMATHWDW *, unsigned long );
int mathWdwDup       ( struct SMATHWDW *, struct SMATHWDW *, unsigned long );
int mathWdwFree      ( struct SMATHWDW *, unsigned long );
int mathWdwInit      ( struct SMATHWDW *, unsigned long );
int mathWdwInside    ( struct SMATHWDW *, double[], int, unsigned long );
int mathWdwInspect   ( struct SMATHWDW *, unsigned long );
int mathWdwOr        ( struct SMATHWDW *, struct SMATHWDW *, struct SMATHWDW *, unsigned long );
int mathWdwPntAdd    ( struct SMATHWDW *, double [], unsigned long );

int mathWdwRectFromPoly( struct SMATHWDW *, unsigned long );

int mathWdwRectLineSect( struct SMATHWDW *, double[], double[], double[], int, unsigned long );

int mathWdwRot       ( struct SMATHWDW *, double[MATHWDWDIM][MATHWDWDIM], struct SMATHWDW *, unsigned long );
int mathWdwShift     ( struct SMATHWDW *, double[MATHWDWDIM], struct SMATHWDW *, unsigned long );

struct SMATHWDW *_mathWdwTest( unsigned long );

                                      /* error handling */
enum { ERRMATHWDW_SYS=1,
       ERRMATHWDW_BADADDR,
       ERRMATHWDW_BADNDIM,
       ERRMATHWDW_DATASKIP,
};
#define MATHWDW_ERROR     0x80000000
#define MATHWDW_WARN      0x40000000
#define MATHWDW_ERRORMASK 0xF0000000
char *mathWdwStrError( int );

#endif

