#ifndef UFILEMEM_H
#define UFILEMEM_H
/*
   Definitions for file/memory interface

   Copyright  2002-2003 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32

   $Log: ufilemem.h,v $
   Revision 1.1.2.2  2005/03/11 18:31:53  kraemer
   MacOS X sync

   Revision 1.1.2.1  2005/01/31 20:40:36  kraemer
   *** empty log message ***


*/

#include <sys/types.h>
#include <sys/stat.h>

#if defined(AMIGA)
 
#include <dos.h>
#include <clib/dos_protos.h>
#include <proto/exec.h>
#include <exec/memory.h>
 
#elif defined(__EMX__)

#else

#include <sys/mman.h>
#include <fcntl.h>

#endif

#include "ummap.h"

                                      /* */
#define UFILEMEM_ERRORMASK 0xFF000000
#define UFILEMEM_ERROR     0x80000000
#define UFILEMEM_WARNING   0x40000000

#define UFILEMEM_MASKFUNC   0x00FF0000/* function specific flags */

#define UFILEMEM_DEBUG  0x80000000    /* general check flag */
#define UFILEMEM_QUIET  0x10000000    /* suppress error messages */
#define UFILEMEM_NOMSG  UFILEMEM_QUIET
                                      /*  */ 
struct SFILEMEM {
   unsigned long ulFlag;
   unsigned long ulFileSize;
   unsigned long ulFileType;          /* compression */ 
   FILE *fp;                          /* stream for (un)compress */
   void *pv; unsigned long ulv;       /* uncompressed data: addr, length
                                         (might be reference to mmap)  
                                      */   
   char **pvcLines;                   /* text files: line pointers */
   int   *nvcLines;                   /* text files: line length */
   struct SUMMAP sMM;
};
                                      /* compression types */
enum { 
   UFILEMEM_PLAIN=0, UFILEMEM_COMPRESS, UFILEMEM_GZIP, UFILEMEM_ZIP, UFILEMEM_LHA, 
};
                                      /* flag bits */
#define UFILEMEM_PVALLOC    0x00000001  /* pv allocated */
#define UFILEMEM_LINESTRDUP 0x00000002  /* line contents by strdup() */
                                      /* mode flags */
#define UFILEMEM_READ   0x00000001    /* file open for read */
#define UFILEMEM_WRITE  0x00000002    /* file open for write */
#define UFILEMEM_APPEND 0x00000004    /* file open for append */
#define UFILEMEM_STREAM 0x00000010    /* file open for stream output */

                                      /* function action flags */
#define UFILEMEMCLOSE_FILEONLY 0x00010000                                       
/*

  uFileMemClose   ( fm, flag ) 
                  o close files, release related resources
                3 o to test
                    x A
                    x O
                    o X
                  o calls
                    o uMMapClose() 

  uFileMemCopy    ( fmto, fmfr, flag )
                  o plain copy, no duplicate, account for linked lists 
                5 o to test
                    x A
                    x O
                    o X
                  o calls

  uFileMemFree    ( fm, flag ) 
                4 o to test
                    x O
                    o X
                    x A 
                  o calls
                    o uFileMemInit()
                    o uFileMemClose()
                    o uMMapFree()

  uFileMemInit    ( fm, flag ) 
                2 o to test
                    x A
                    x O
                    o X
                  o calls
                    o uMMapInit() 

  uFileMemInspect ( fm, flag ) 
                1 o to test
                    x A
                    x O
                    o X
                  o calls
                    o uMMapInspect() 

  uFileMemLines   ( fm, flag )
                  o for text files: establish line arrays 
                8 o to test
                    o A
                    x O
                    o X
                  x support 0x1A (CTRL-Z ?)
                  x inplace compress of extra \r, 0x1A
                  o calls

  uFileMemOpen    ( fm, filename, mode, flag )
                  o open a file, prepare for read
                6 o to test
                    o A
                    x O
                    o X
                  ! no => use given filename over internal one
                  o mode: read, write, append
                  x remember compression mode
                  o all compressions on all platforms
                       .lha .Z .gz .zip
                    A:  
                    X:   
                    O:      +  +   +     
                    o get zip, compress, gzip, unzip, uncompress, gunzip
                    o executables + API
                  o calls
                    o uMMapOpen()
                    o uMMapClose()
                    o uFileMemClose()

  uFileMemRead    ( fm, flag )
                  o "read" (uncompress) file, sets fm->pv  
                7 o to test
                    x A
                    x O
                    o X  
                  o fread(): efficient buffer size ?
                  o calls

  uFileMemWrite   ( fm, flag )
                  o write data from given fm->pv or lines arrays
                9 o to test
                    x O 
                    o X
                    o A 
                  o keep fp valid    
                  o calls
*/
                                      /* function prototypes */
int uFileMemClose  ( struct SFILEMEM *, unsigned long );
int uFileMemCopy   ( struct SFILEMEM *, struct SFILEMEM *, unsigned long );
int uFileMemFree   ( struct SFILEMEM *, unsigned long );
int uFileMemInit   ( struct SFILEMEM *, unsigned long );
int uFileMemInspect( struct SFILEMEM *, unsigned long );
int uFileMemLines  ( struct SFILEMEM *, unsigned long );
int uFileMemOpen   ( struct SFILEMEM *, const char *, unsigned long, unsigned long );
int uFileMemRead   ( struct SFILEMEM *, unsigned long );
int uFileMemWrite  ( struct SFILEMEM *, unsigned long );

struct SFILEMEM *_uFileMemTest( unsigned long );

/*
   --- error handlind
*/
char *uFileMemStrError( int );

//int uMMapError  ( struct SCMDERR *, unsigned long );

enum { ERRUFILEMEM_SYS=1, 
       ERRUFILEMEM_BADADDR,
       ERRUFILEMEM_NOBUFFER,
       ERRUFILEMEM_BADFILENAME,
       ERRUFILEMEM_BADFILESIZE,
       ERRUFILEMEM_STATFAILED, 
       ERRUFILEMEM_POPENFAILED, 
       ERRUFILEMEM_FOPENFAILED, 
       ERRUFILEMEM_UMMAPFAILED, 
       ERRUFILEMEM_NOMAP, 
};

#endif
